/*
	File:		som_ListPart.cpp

	Contains:	Sample Part's SOM based part public interface implementation

	Written by:	Steve Smith
	
	Copyright:	 1995 by Apple Computer, Inc., all rights reserved.
	
	----------------------------------------------------------------
	
	Notes:		ListPart is implemented using a "wrapped" class strategy.
				This means that the SOM class is just a smart delegator and
				the real implementation lives in a C++ class. The reason for
				doing this is twofold. First, this strategy all but elimintates
				the need for you to have to modify the IDL file and run the
				SOM compiler/emitter. Secondly, given that all code changes
				will occur in the C++ class, build turn around time is reduced
				to however fast the compiler can crunch code.
				
				This is not to say that implementing pure SOM classes isn't
				possible, nor necessary. There will be objects (ie. Extensions)
				which need to be exported for other parts to use which will need
				to be written in IDL and SOM implementation files.
*/

// Notification that this is a SOM source file
#define ACF_DevServ_som_ListPart_Class_Source

// define underscore field names (ie. _fSelf)
#define VARIABLE_MACROS

// -- Compiler/Preprocessor Switches --

#ifndef _COMPILERDEFS_
#include "CompDefs.h"
#endif

// -- OpenDoc Utilities --

#ifndef _EXCEPT_
// Exceptions define several important macros (ie. CHECKENV)
// which are used in the SOM method dispatch glue. If Except.h
// is not included early enough, exceptions may not be thrown
// correctly when returning from a SOM method with the "ev" parameter set.
#include <Except.h>
#endif

// -- ListPart Includes --

#ifndef SOM_ACF_DevServ_som_ListPart_xih
#include "som_ListPart.xih"
#endif

#ifndef _LISTPART_
#include "ListPart.h"
#endif

#ifndef _LISTPARTDEF_
#include "ListPartDef.h"
#endif

// -- OpenDoc Includes --

#ifndef _ODTYPES_
#include <ODTypes.h>
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#pragma segment somListPart

SOM_Scope void  SOMLINK som_ListPart__ShowMe(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		short theLine)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ShowMe");

	SOM_TRY
		_fPart->ShowMe(ev,facet,theLine);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope short  SOMLINK som_ListPart__GetNbLines(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetNbLines");

	short result;

	SOM_TRY
		result = _fPart->GetNbLines(ev);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_ListPart__SetNbLines(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		short newNbLines)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__SetNbLines");

	SOM_TRY
		_fPart->SetNbLines(ev,newNbLines);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope short  SOMLINK som_ListPart__GetSel(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetSel");

	short result;

	SOM_TRY
		result = _fPart->GetSel(ev);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

/*
 *  to override
 */

SOM_Scope void  SOMLINK som_ListPart__SetSel(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		short theLine)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__SetSel");

	SOM_TRY
		_fPart->SetSel(ev,facet,theLine);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

/*
 *  to override
 */

SOM_Scope ODISOStr  SOMLINK som_ListPart__GetTheRealPartKind(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetTheRealPartKind");

	ODISOStr result;

	SOM_TRY
		result = _fPart->GetTheRealPartKind(ev);
	SOM_CATCH_ALL
		result = "";
	SOM_ENDTRY
	
	return result;
}

SOM_Scope ODSLong  SOMLINK som_ListPart__OverrideBeginUsingLibraryResources(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__OverrideBeginUsingLibraryResources");

	ODSLong result;

	SOM_TRY
		result = _fPart->OverrideBeginUsingLibraryResources(ev);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_ListPart__OverrideEndUsingLibraryResources(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODSLong ref)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__OverrideEndUsingLibraryResources");

	SOM_TRY
		_fPart->OverrideEndUsingLibraryResources(ev,ref);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__SetUpGraphics(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		void* theGWorld)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__SetUpGraphics");

	SOM_TRY
		_fPart->SetUpGraphics(ev,(GWorldPtr)theGWorld);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FillCell(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		short theLine,
		Rect* theRect)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FillCell");

	SOM_TRY
		_fPart->FillCell(ev,theLine,theRect);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FillHilCell(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		short theLine,
		Rect* theRect)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FillHilCell");

	SOM_TRY
		_fPart->FillHilCell(ev,theLine,theRect);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ClickInActive(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		ODEventData* event,
		Rect* theRect)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ClickInActive");

	SOM_TRY
		_fPart->ClickInActive(ev,facet,event,theRect);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__CloseOpenedCell(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CloseOpenedCell");

	SOM_TRY
		_fPart->CloseOpenedCell(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__IdleOpened(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__IdleOpened");

	SOM_TRY
		_fPart->IdleOpened(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope short  SOMLINK som_ListPart__KeyInActive(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		ODEventData* event)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__KeyInActive");

	short result;

	SOM_TRY
		result = _fPart->KeyInActive(ev, facet, event);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope short  SOMLINK som_ListPart__KeyShortCut(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		char theChar)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__KeyShortCut");

	short result;

	SOM_TRY
		result = _fPart->KeyShortCut(ev,theChar);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_ListPart__GotDoubleClick(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		short theLine)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GotDoubleClick");

	SOM_TRY
		_fPart->GotDoubleClick(ev,facet,theLine);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ExternalizeListData(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnit* storageUnit)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ExternalizeListData");

	SOM_TRY
		_fPart->ExternalizeListData(ev,storageUnit);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__InternalizeListData(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnit* storageUnit)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__InternalizeListData");

	SOM_TRY
		_fPart->InternalizeListData(ev,storageUnit);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__SetUpListData(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnit* storageUnit)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__SetUpListData");

	SOM_TRY
		_fPart->SetUpListData(ev,storageUnit);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__InitializeListData(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		short* pNbLines,
		short* pLineHeight,
		short* pLineWidth,
		short* pLineDepth,
		short* pKind,
		short* pAutoThumb,
		short* pWantKey,
		short* pListIndex,
		short* pSel,
		char** pMul)
{
    ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__InitializeListData");

	SOM_TRY
		_fPart->InitializeListData(ev,pNbLines,pLineHeight,pLineWidth,pLineDepth,pKind,pAutoThumb,pWantKey,pListIndex,pSel,pMul);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

//====================================================================
// som_ListPart
//====================================================================

SOM_Scope void  SOMLINK som_ListPart__somInit(ACF_DevServ_som_ListPart *somSelf)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__somInit");

	// somInit and somUninit methods behave like C++ constructors in that the
	// inherited methods are called automatically. Thus, there is no need to
	// call the parent class' somInit or somUninit.
	
	// There is also no need to set instance variables to zero/NULL
	// since SOM guarantees that a newly constructed object is zeroed.
	
	// Lastly, it is not valid to perform any operation here which may throw
	// an exception.
}

SOM_Scope void  SOMLINK som_ListPart__somUninit(ACF_DevServ_som_ListPart *somSelf)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__somUninit");

	delete _fPart;
}

SOM_Scope void  SOMLINK som_ListPart__InitPart(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__InitPart");

	SOM_TRY
		// Create the real C++ ListPart.		
		_fPart = new ListPart;

		ACF_DevServ_som_ListPart_parent_ODPart_InitPart(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPart(ev,storageUnit,partWrapper,somSelf);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__InitPartFromStorage(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnit* storageUnit,
		ODPart* partWrapper)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__InitPartFromStorage");

	SOM_TRY
		// Create the real C++ ListPart.		
		_fPart = new ListPart;

		ACF_DevServ_som_ListPart_parent_ODPart_InitPartFromStorage(somSelf,ev,storageUnit,partWrapper);
		_fPart->InitPartFromStorage(ev,storageUnit,partWrapper,somSelf);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__Externalize(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Externalize");

	SOM_TRY
		ACF_DevServ_som_ListPart_parent_ODPart_Externalize(somSelf,ev);
		_fPart->Externalize(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODSize  SOMLINK som_ListPart__Purge(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODSize size)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Purge");

	ODSize result;

	SOM_TRY
		result = _fPart->Purge(ev,size);
	SOM_CATCH_ALL
		result = 0;
	SOM_ENDTRY
	
	return result;
}

SOM_Scope void  SOMLINK som_ListPart__Release(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Release");

	SOM_TRY
		ACF_DevServ_som_ListPart_parent_ODPart_Release(somSelf,ev);
		_fPart->Release(ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ReleaseAll(ACF_DevServ_som_ListPart *somSelf, Environment *ev)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ReleaseAll");

	SOM_TRY
		_fPart->ReleaseAll(ev);
		ACF_DevServ_som_ListPart_parent_ODPart_ReleaseAll(somSelf,ev);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODExtension*  SOMLINK som_ListPart__AcquireExtension(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODType extensionName)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetExtension");

	ODExtension* result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = ACF_DevServ_som_ListPart_parent_ODPart_AcquireExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODBoolean  SOMLINK som_ListPart__HasExtension(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODType extensionName)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__HasExtension");

	ODBoolean result;

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
		result = ACF_DevServ_som_ListPart_parent_ODPart_HasExtension(somSelf,ev,extensionName);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_ListPart__ReleaseExtension(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODExtension* extension)
{
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ReleaseExtension");

	SOM_TRY
		// We don't support any extensions, but one of our parent classes might;
	    ACF_DevServ_som_ListPart_parent_ODPart_ReleaseExtension(somSelf,ev,extension);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FulfillPromise(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODStorageUnitView* /*promiseSUView*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FulfillPromise");
}

SOM_Scope void  SOMLINK som_ListPart__DropCompleted(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODPart* ,
		ODDropResult dropResult)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DropCompleted");
}

SOM_Scope ODDragResult  SOMLINK som_ListPart__DragEnter(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODDragItemIterator* /*dragInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DragEnter");

	return kODFalse;
}

SOM_Scope ODDragResult  SOMLINK som_ListPart__DragWithin(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODDragItemIterator* /*dragInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DragWithin");

	return kODFalse;
}

SOM_Scope void  SOMLINK som_ListPart__DragLeave(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DragLeave");
}

SOM_Scope ODDropResult  SOMLINK som_ListPart__Drop(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODDragItemIterator* /*dropInfo*/,
		ODFacet* /*facet*/,
		ODPoint* /*where*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Drop");

	return kODDropFail;
}

SOM_Scope ODBoolean  SOMLINK som_ListPart__RevealFrame(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*revealShape*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RevealFrame");

	return kODFalse;
}

SOM_Scope ODBoolean  SOMLINK som_ListPart__EditInLinkAttempted(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
    ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__EditInLinkAttempted");

    return kODFalse;
}

SOM_Scope void  SOMLINK som_ListPart__EmbeddedFrameUpdated(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame,
		ODUpdateID change)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__EmbeddedFrameChanged");
}

SOM_Scope void  SOMLINK som_ListPart__EmbeddedFrameSpec(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODObjectSpec* /*spec*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__EmbeddedFrameSpec");
}

SOM_Scope void  SOMLINK som_ListPart__ContainingPartPropertiesUpdated(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnit* propertyUnit)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ContainingPartPropertiesChanged");
}

SOM_Scope ODEmbeddedFramesIterator*  SOMLINK som_ListPart__CreateEmbeddedFramesIterator(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CreateEmbeddedFramesIterator");

	// Scripting know if you part supports embedding by calling this method.
	// Since we dont' support embedding, we let ODPart handle it. This is the
	// only "unimplemented" method that we really have to handle.
	ACF_DevServ_som_ListPart_parent_ODPart_CreateEmbeddedFramesIterator(somSelf,ev,frame);
	return kODNULL;
}

SOM_Scope void  SOMLINK som_ListPart__DisplayFrameAdded(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DisplayFrameAdded");

	SOM_TRY
		_fPart->DisplayFrameAdded(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__DisplayFrameRemoved(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DisplayFrameRemoved");

	SOM_TRY
		_fPart->DisplayFrameRemoved(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__DisplayFrameConnected(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DisplayFrameConnected");

	SOM_TRY
		_fPart->DisplayFrameConnected(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__DisplayFrameClosed(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DisplayFrameClosed");

	SOM_TRY
		_fPart->DisplayFrameClosed(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__AttachSourceFrame(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame,
		ODFrame* sourceFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__AttachSourceFrame");

	SOM_TRY
		_fPart->AttachSourceFrame(ev,frame,sourceFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FrameShapeChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FrameShapeChanged");

	SOM_TRY
		_fPart->FrameShapeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ViewTypeChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ViewTypeChanged");

	SOM_TRY
		_fPart->ViewTypeChanged(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__PresentationChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__PresentationChanged");
}

SOM_Scope void  SOMLINK som_ListPart__SequenceChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__SequenceChanged");
}

SOM_Scope void  SOMLINK som_ListPart__ClonePartInfo(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODDraftKey key,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView,
		ODFrame* scopeFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ClonePartInfo");

	SOM_TRY
		_fPart->ClonePartInfo(ev,key,partInfo,storageUnitView,scopeFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__WritePartInfo(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODInfoType partInfo,
		ODStorageUnitView* storageUnitView)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__WritePartInfo");

	SOM_TRY
		_fPart->WritePartInfo(ev,partInfo,storageUnitView);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODInfoType  SOMLINK som_ListPart__ReadPartInfo(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame,
		ODStorageUnitView* storageUnitView)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ReadPartInfo");

	ODInfoType result;

	SOM_TRY
		result = _fPart->ReadPartInfo(ev,frame,storageUnitView);
	SOM_CATCH_ALL
		result = kODNULL;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODID  SOMLINK som_ListPart__Open(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Open");

	ODID result;

	SOM_TRY
		result = _fPart->Open(ev,frame);
	SOM_CATCH_ALL
		result = kODNULLID;
	SOM_ENDTRY

	return result;
}

SOM_Scope ODFrame*  SOMLINK som_ListPart__RequestEmbeddedFrame(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* containingFrame,
		ODFrame* baseFrame,
		ODShape* frameShape,
		ODPart* embeddedPart,
		ODTypeToken viewType,
		ODTypeToken presentation,
		ODBoolean isOverlaid)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RequestEmbeddedFrame");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_ListPart__RemoveEmbeddedFrame(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RemoveEmbeddedFrame");
}

SOM_Scope ODShape*  SOMLINK som_ListPart__RequestFrameShape(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/,
		ODShape* /*frameShape*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RequestFrameShape");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_ListPart__UsedShapeChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*embeddedFrame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__UsedShapeChanged");
}

SOM_Scope ODStorageUnit*  SOMLINK som_ListPart__AcquireContainingPartProperties(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetContainingPartProperties");

	return kODNULL;
}

SOM_Scope ODShape*  SOMLINK som_ListPart__AdjustBorderShape(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* /*embeddedFacet*/,
		ODShape* /*shape*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__AdjustBorderShape");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_ListPart__FacetAdded(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FacetAdded");

	SOM_TRY
		_fPart->FacetAdded(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FacetRemoved(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FacetRemoved");

	SOM_TRY
		_fPart->FacetRemoved(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__CanvasChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* /*facet*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CanvasChanged");
}

SOM_Scope void  SOMLINK som_ListPart__GeometryChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		ODBoolean clipShapeChanged,
		ODBoolean externalTransformChanged)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GeometryChanged");

	SOM_TRY
		_fPart->GeometryChanged(ev,facet,clipShapeChanged,externalTransformChanged);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__Draw(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet,
		ODShape* invalidShape)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__Draw");

	SOM_TRY
		_fPart->Draw(ev,facet,invalidShape);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__CanvasUpdated(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODCanvas* /*canvas*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CanvasUpdated");
}

SOM_Scope void  SOMLINK som_ListPart__HighlightChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFacet* facet)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__HighlightChanged");

	SOM_TRY
		_fPart->HighlightChanged(ev,facet);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODULong  SOMLINK som_ListPart__GetPrintResolution(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__GetPrintResolution");

	return kMinImagingResolution;
}

SOM_Scope ODLinkSource*  SOMLINK som_ListPart__CreateLink(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODByteArray* /*data*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CreateLink");

	return kODNULL;
}

SOM_Scope void  SOMLINK som_ListPart__LinkUpdated(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODLink* link,
		ODUpdateID change)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__LinkUpdated");
}

SOM_Scope void  SOMLINK som_ListPart__RevealLink(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODLinkSource* /*linkSource*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RevealLink");
}

SOM_Scope void  SOMLINK som_ListPart__LinkStatusChanged(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* /*frame*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__LinkStatusChanged");
}

SOM_Scope ODBoolean  SOMLINK som_ListPart__BeginRelinquishFocus(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__BeginRelinquishFocus");

	ODBoolean result;

	SOM_TRY
		result = _fPart->BeginRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_ListPart__CommitRelinquishFocus(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CommitRelinquishFocus");

	SOM_TRY
		_fPart->CommitRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__AbortRelinquishFocus(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame,
		ODFrame* proposedFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__AbortRelinquishFocus");

	SOM_TRY
		_fPart->AbortRelinquishFocus(ev,focus,ownerFrame,proposedFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FocusAcquired(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FocusAcquired");

	SOM_TRY
		_fPart->FocusAcquired(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__FocusLost(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeToken focus,
		ODFrame* ownerFrame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__FocusLost");

	SOM_TRY
		_fPart->FocusLost(ev,focus,ownerFrame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__CloneInto(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODDraftKey key,
		ODStorageUnit* toSU,
		ODFrame* scope)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__CloneInto");

	SOM_TRY
		ACF_DevServ_som_ListPart_parent_ODPart_CloneInto(somSelf,ev,key,toSU,scope);
		_fPart->CloneInto(ev,key,toSU,scope);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ExternalizeKinds(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODTypeList* kindset)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ExternalizeKinds");

	SOM_TRY
		_fPart->ExternalizeKinds(ev,kindset);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__ChangeKind(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODType kind)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__ChangeKind");

	SOM_TRY
		_fPart->ChangeKind(ev,kind);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope ODBoolean  SOMLINK som_ListPart__HandleEvent(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODEventData* event,
		ODFrame* frame,
		ODFacet* facet,
		ODEventInfo* eventInfo)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__HandleEvent");

	ODBoolean result;

	SOM_TRY
		result = _fPart->HandleEvent(ev,event,frame,facet,eventInfo);
	SOM_CATCH_ALL
		result = kODFalse;
	SOM_ENDTRY

	return result;
}

SOM_Scope void  SOMLINK som_ListPart__AdjustMenus(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODFrame* frame)
{
	ACF_DevServ_som_ListPartData *somThis = ACF_DevServ_som_ListPartGetData(somSelf);
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__AdjustMenus");

	SOM_TRY
		_fPart->AdjustMenus(ev,frame);
	SOM_CATCH_ALL
	SOM_ENDTRY
}

SOM_Scope void  SOMLINK som_ListPart__UndoAction(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__UndoAction");
}

SOM_Scope void  SOMLINK som_ListPart__RedoAction(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODActionData* /*actionState*/)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__RedoAction");
}

SOM_Scope void  SOMLINK som_ListPart__DisposeActionState(ACF_DevServ_som_ListPart *somSelf, Environment *ev,
		ODActionData* ,
		ODDoneState doneState)
{
	ACF_DevServ_som_ListPartMethodDebug("ACF_DevServ_som_ListPart","som_ListPart__DisposeActionState");
}
