/*
	File:		AppleEventHandling.h

	Contains:	Minimalist support for the required and Display Manager suites
				To really support AppleScript, well do more work in here.

	Written by: Dave Falkenburg

	Copyright:	 1993-1994 by Dave Falkenburg, all rights reserved.

	Change History (most recent first):	 

		 <7>	12/15/94	DRF		Added ChooseApplicationAsAEtarget.
		 <6>	11/23/94	DRF		InstallAppleEventHandlers is now called InitAppleEventRoutines.
									Also added a useful global: gThisProcessDesc for self targetting
									of AppleEvents.
		 <5>	11/16/94	DRF		Add StandardAEIdleProc for people who like using AESend with
									kAEWaitReply.
		 <4>	11/12/94	DRF		AppConditionals.h -> BuildConditionals.h
		 <2>	  9/4/94	DRF		Added Text Services AppleEvent handlers.
 */

#ifndef		_APPLEEVENTHANDLING_
#define		_APPLEEVENTHANDLING_

#ifndef		__APPLEEVENTS__
#include	<AppleEvents.h>
#endif

#ifndef		__OCESTANDARDMAIL__
#include	<OCEStandardMail.h>
#endif


void				InitAppleEventRoutines(void);
OSErr				CheckAppleEventForMissingParams(AppleEvent * theAppleEvent);
OSErr				ChooseApplicationAsAETarget(AEAddressDesc *targetDesc, StringPtr prompt);

OSErr				SendOpenDocumentToSelf(LetterDescriptor * theDocument);
OSErr				SendQuitApplicationToSelf();


extern AEIdleUPP	StandardAEIdleUPP;
extern AEDesc		gThisProcessDesc;

//	Handlers for the Required Suite:

typedef OSErr		(*EachDocumentProcPtr)(LetterDescriptor *aFile,void * param);
OSErr				ForEachDocumentInList(AEDescList documentList,EachDocumentProcPtr documentProc,void * documentParam);

pascal OSErr		HandleOpenApplication(AppleEvent * theAppleEvent,AppleEvent * reply, long refCon);
pascal OSErr		HandleOpenDocuments(AppleEvent * theAppleEvent,AppleEvent * reply, long refCon);
pascal OSErr		HandlePrintDocuments(AppleEvent * theAppleEvent,AppleEvent * reply, long refCon);
pascal OSErr		HandleQuitApplication(AppleEvent * theAppleEvent,AppleEvent * reply, long refCon);


//	Display Manager:

pascal OSErr	HandleSystemConfigNotice(AppleEvent *theAppleEvent,AppleEvent *reply,long refCon);


#if	qInlineInputAware

//	Text Services Manager:

pascal OSErr	HandleTextServicesUpdateActiveInputArea(AppleEvent *theAppleEvent,AppleEvent *reply,long refCon);
pascal OSErr	HandleTextServicesPos2Offset(AppleEvent *theAppleEvent,AppleEvent *reply,long refCon);
pascal OSErr	HandleTextServicesOffset2Pos(AppleEvent *theAppleEvent,AppleEvent *reply,long refCon);

#endif

#endif
