/*
	File:		MenuBar.h

	Contains:	A MenuBar class deliberately designed to resemble
				ODMenuBar from OpenDoc. Because it is really just
				a table of menu commands, we depend on our sorted
				dynamic array base class to maintain mappings.

	Written by: Dave Falkenburg

	Copyright:	 1994-95 by Dave Falkenburg, all rights reserved.

	Change History (most recent first):

		 <3>	 1/24/95	DRF		Finished up work on CMNUs for next major release.
		 <2>	 1/20/95	DRF		Added MenuBarResource type, revised some methods.
		 <1>	  1/3/95	DRF		First checked in.
 */
#ifndef	_MENUBAR_
#define	_MENUBAR_

#include <Menus.h>
#include "SortedDynamicArray.h"

//	bare types are dangerous, eh?

typedef	SInt16	MenuID;
typedef	UInt16	MenuItemID;
typedef	SInt16	MenuBarTemplateID;
typedef	UInt32	MenuCommandID;

//	Some types which should probably be defined in <Menus.h>
//	NOTE: These must be aligned on 2-byte boundaries
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=mac68k
#endif

//	Kinda strange that NONE of the toolbox headers define
//	the ON-DISK representations of their template resources, eh?

typedef	struct	MenuBarResource	MenuBarResource;

struct	MenuBarResource
	{
	UInt16	numberOfMenus;
	SInt16	menuIDList[1];
	};


//	CMNU Resources look just like 'MENU' resources, except that
//	each item is appended with an extra (2-byte aligned) longword.
//
//	They are evil because menu items always start out screwed up
//	because some bright individual decided to START the record
//	with the variable-sized field!.
//
//	In the long run this is still OK, because ResEdit and Resourceror
//	already create and edit CMNU resources. All the evil is confined
//	within parts of Sprocket you dont have to modify.

typedef	struct	CMNUResource	CMNUResource;
typedef	struct	CMNUItemData	CMNUItemData;


//	CW6 didnt like having this below the CMNUResource definition,
//	even though CW5 didnt complain:

struct	CMNUItemData
	{
	Str255				itemString;	//	dont you hate structures that start out misalinged!
	Byte				itemIcon;
	Byte				itemCmd;
	Byte				itemMark;
	Style				itemStyle;
	MenuCommandID		itemCommand;
	};


struct CMNUResource
	{
	short				menuID;
	short				menuWidth;
	short				menuHeight;
	Handle				menuProc;
	long				enableFlags;
	Str255				menuTitle;
	CMNUItemData		menuData[1];	//	here begins evil!
	};


//	Restore default alignment
#if PRAGMA_ALIGN_SUPPORTED
#pragma options align=reset
#endif


enum
	{
	kNoMenuID			=	0,
	kNoMenuItemID		=	0,
	kNoMenuCommandID	=	0
	};


struct	MenuMapping
	{
	MenuID			fMenu;
	MenuItemID		fItem;
	MenuCommandID	fCommand;		//	the command ID, which must be unique
	};


class	TMenuItemTable : public TSortedDynamicArray
	{
	virtual	CompareResult	Compare(ArrayElementPtr element1,
									ArrayElementPtr	element2);
	};


class	TMenuCommandTable : public TSortedDynamicArray
	{
	virtual	CompareResult	Compare(ArrayElementPtr element1,
									ArrayElementPtr	element2);
	};


class	TMenuBar
	{
public:
	//	Resource ('MBAR' and 'CMNU') Utilities
	
	OSErr					GetNewMenuBar(short whichMBAR);
	MenuRef					GetMenuFromCMNU(short whichMenu);
	
	//	Menu command mapping functions

	MenuCommandID			GetCommand(MenuID menu, MenuItemID item);
	void					GetMenuAndItem(MenuCommandID commandNum, MenuID * returnedMenu, MenuItemID * returnedItem);
	
	OSErr					RegisterCommand(MenuCommandID commandNum, MenuID menu, MenuItemID item);
	OSErr					UnregisterCommand(MenuCommandID commandNum);
	
	//	Menu enable/disable routines for menu items

	void					EnableCommand(MenuCommandID commandNum, Boolean enable);
	void					EnableAndCheckCommand(MenuCommandID commandNum, Boolean enable, Boolean check);

	void					GetItemString(MenuCommandID commandNum,StringPtr itemString);
	void					SetItemString(MenuCommandID commandNum,StringPtr itemString);

	//	helpful utility functions

	static void				HideMenuBar();
	static void				ShowMenuBar();

	static void				EnterModalState();
	static void				ExitModalState();

private:
	//	"globals"
	
	static Boolean			fgMenuBarHidden;

	//	mapping tables

	TMenuCommandTable		fCommandTable;
	TMenuItemTable			fMenuItemTable;

	//	internal methods
	
	MenuHandle				GetMenuHandleAndItemFromCommand(MenuCommandID commandNum, MenuID *menu,MenuItemID *item);
	};

#endif
