/*
	File:		SprocketMain.cp

	Contains:	The guts of a Macintosh application.

	Written by: Dave Falkenburg and many other SmartFriends

	Copyright:	 1993-1995 by Dave Falkenburg, all rights reserved.

	Change History (most recent first):
	 
		<17>	 1/24/95	DRF		Added support for HandleMenuSelection. When building with GX
									debugging on stop requiring a special hacked file by #including
									the official C file here.
		<16>	 1/20/95	DRF		Use our own special version of GetNewMenuBar which should
									automatically register menu commands found in CMNU resources.
		<15>	  1/3/95	DRF		Add Nitins changes for Drag handling: Call ClickAndDrag method
									if the mouse starts moving after a click.
		<14>	  1/3/95	DRF		Overhauled menu handling: HandleMenuSelection is now part of Sprocket.
									Added gMenuBar, a TMenuBar object which handles menu <-> menu
									command mappings. Start calling AdjustMenusBeforeMenuSelection.
		<13>	 12/6/94	DRF		kUnresolvedSymbolAddress isnt a Ptr in the latest universal
									interfaces.
		<12>	11/23/94	DRF		InstallAppleEventHandlers is now called InitAppleEventRoutines.
		<11>	11/16/94	DRF		Added an explicit #include <Traps.h> for latest universal
									headers. Also killed off a CFront warning.
		<10>	11/12/94	DRF		Revised QuickDrawGX initialization based on changes made by Jon
									Summer. We now use a 'gasz' resource to setup the size of the
									graphics heap. Also we are now always TSM aware, not just when
									TSMTE is around.
		 <9>	 11/8/94	DRF		Rolled in LDRs TSM fixes. Add fonts to the Fonts menu if it
									exists. Also, stop openning up the preferences file here.
		 <8>	 9/27/94	DRF	 	AppLib.h is now Sprocket.h, also fixed scrap coercion routines
		 							and eliminated HandleUpdate.
		 <7>	  9/9/94	DRF		Reordered headers and removed redundant #includes.
									Conditionalized AOCE support. Also made changes to
									TSMEventWrapper to be more like the code in InlineInputSample.
		 <6>	  9/8/94	DRF		Add HiliteMenu(0) calls after calls to menu handling code
									because TSMMenuSelect doesnt do it. Also reorganized some GX
									stuff: allocate and release a graphicsclient (Cam claims this
									works on his machine), and turn on all the graphics errors and
									notices if qDebug is set.
		 <5>	  9/7/94	DRF		Rearrange YieldToAnyThread calls to make quitting faster.
		 <4>	  9/4/94	DRF		Rearrange local variables in HandleMouseDown.
		 <3>	  9/1/94	DRF		Stop including "MailableWindow.h" here.
		 <2>	  9/1/94	DRF		More fixes for inline input, but were not really done yet.
 */

#include "Sprocket.h"

#include <limits.h>		//	For LONG_MAX

#include <Fonts.h>
#include <Desk.h>
#include <GestaltEqu.h>
#include <DiskInit.h>
#include <Threads.h>
#include <CodeFragments.h>	//	for kUnresolvedSymbolAddress
#include <Traps.h>

#if	qInlineInputAware
#include <TextServices.h>
#include <TSMTE.h>
#endif

#if	qUseQuickDrawGX
#include <graphics macintosh.h>
#include <graphics routines.h>
#include <graphics libraries.h>
#include <PrintingManager.h>
#if	qUseQuickDrawGXDebugging
#include <graphics debug library.c>
#endif
#endif

#include "StandardMenus.h"
#include "Window.h"
#include "SplashWindow.h"
#include "AppleEventHandling.h"


//	Function Prototypes

void	main(void);
void	MainEventLoop(void);

void	HandleMouseDown(TWindow * topWindowObj, EventRecord * anEvent);
void	HandleClose(WindowRef aWindow);
void	HandleMenu(TWindow * topWindow, long menuCode);


//	Globals

Boolean				gDone = false;
TMenuBar *			gMenuBar = nil;
TSplashWindow *		gSplashWindow = nil;

Boolean				gHasColorQuickdraw = false;
Boolean				gHasThreadManager = false;
Boolean				gHasDragManager = false;
Boolean				gHasAppleScript = false;
Boolean				gHasDisplayManager = false;

GrafPtr				gWindowManagerPort;
Rect				gDeskRectangle;
RgnHandle			gMouseRegion = nil;

short				gPreferencesRsrcRefNum;

#if	qAOCEAware
Boolean				gHasAOCE = false;
#endif

#if	qInlineInputAware
Boolean				gHasTextServices = false;
Boolean				gHasTSMTE = false;

Boolean				TSMEventWrapperForKotoeri(EventRecord *anEvent);

#endif

#if	qUseQuickDrawGX
Boolean				gHasQuickDrawGX = false;
long				gQuickDrawGXVersion = 0;
long				gQuickDrawGXPrintingVersion = 0;
gxGraphicsClient	gQuickDrawGXClient;
#endif


//	Values that can be adjusted by other application code to change
//	the behavior of the MainEventLoop.
//
//	Rules of thumb:
//
//		Increase gXXXRunQuantum (and decrease gXXXSleepQuantum) when:
//			The application has many threads running that need time
//
//		Decrease gXXXRunQuantum when:
//			Sending AppleEvents to other applications
//			Launching other applications
//			Running in the background

unsigned long	gForegroundRunQuantum = 0;
unsigned long	gForegroundSleepQuantum = GetCaretTime();
unsigned long	gBackgroundRunQuantum = 0;
unsigned long	gBackgroundSleepQuantum = LONG_MAX;


//	Globals used to tune the performance of MainEventLoop
//	(assume well be starting in the foreground)

static	unsigned long	gRunQuantum = gForegroundRunQuantum;
static	unsigned long	gSleepQuantum = gForegroundSleepQuantum;

#if GENERATINGPOWERPC
#ifndef	__MWERKS__
QDGlobals	qd;
#endif
#endif

void
main(void)
	{
	long		feature;

	MaxApplZone();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();

	InitGraf(&qd.thePort);	
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);


	if (GetToolTrapAddress(_Unimplemented) == GetOSTrapAddress(_Gestalt))
		FatalErrorAlert(kCoreErrorStrings,kUnsupportedSystemSoftware);

	if (Gestalt(gestaltQuickdrawFeatures,&feature) == noErr)
		gHasColorQuickdraw = ((feature & (1 << gestaltHasColor)) != 0);

	gSplashWindow = new TSplashWindow;	

	//	Check for and Initialize QuickDraw GX early on to avoid heap fragmentation
	
#if	qUseQuickDrawGX
	//	Check for and initialize QuickDrawGX
	if (Gestalt(gestaltGXVersion, &gQuickDrawGXVersion) == noErr)
		{
		if (Gestalt(gestaltGXPrintingMgrVersion, &gQuickDrawGXPrintingVersion) == noErr)
			{
#if GENERATINGCFM
			if ((Ptr) GXEnterGraphics != kUnresolvedSymbolAddress)
#endif
			gHasQuickDrawGX = true;
			}
		}

	if (gHasQuickDrawGX)
		{
		// Initialize the graphics and printing environments.
		//	For additional details see "IM: QD GX Environment & Utilities."
		
		//	We let QuickDraw GX look for a resource  of type 'gasz' with an ID of 0.
		//	GX uses the first long word of that resource as the graphics heap size.
		//	To determine the memory requirements of your graphics client heap see:
		//	IM: QD GX Environment & Utilities pp2-8,2-9
		
		//	NOTE:	If your application does not provide this resource,
		//			QuickDraw GX version 1.0 uses a default size of 600 KB.

		gQuickDrawGXClient = GXNewGraphicsClient(nil, kUseGraphicsSizeResource, kAllowGXToExtendGraphicsHeap);
		if (gQuickDrawGXClient)
			{
			//	NOTE:	GXNewGraphicsClient() does not allocate memory to the heap
			//			until GXEnterGraphics()	is called, so go ahead and call it now
			//			while we have a chance to deal with things nicely.
			//
			// NOTE:	GXEnterGraphics() should only fail if there is not enough memory. 
			
			GXEnterGraphics();

			if (GXGetGraphicsError(nil) == noErr)
				{
#if	qUseQuickDrawGXDebugging
				// If qUseQuickDrawGXDebugging = TRUE, enable on GX Validation and graphics library errors & notices.

				//	NOTES from Jon Summers:
				//	As you increase the amount of validation, drawing speed will SLOW
				//	down due to all of the internal checking. Also, if you play with
				//	settings enough validation seems to become inconsistent

				GXSetValidation(	//	gxInternalValidation	|
									//	gxAllObjectValidation	|
									//	gxApHeapValidation		|
										gxPublicValidation);		// check parameters to public routines

				//	This functionality will only work with the "debugging" version of QuickDraw GX.
				//	If you don't have the debugging version installed, these functions will not work. 

				SetGraphicsLibraryErrors();
				SetGraphicsLibraryNotices();	
#endif
				if (GXInitPrinting() != noErr)		// See IM: QuickDraw GX Printing
					{
					//	We couldnt initialize printing, so clean up pretend GX isnt around
					//	 For safetys sake, well probably want to bail here.
					
					if (gQuickDrawGXClient)
						GXDisposeGraphicsClient(gQuickDrawGXClient);
					gHasQuickDrawGX = false;
					}
				}
			}
		}

	if (gQuickDrawGXClient)
		InitCommonColors();		// Initialize the CommonColors Library.
#if	qRequireQuickDrawGX
	else
		FatalErrorAlert(kCoreErrorStrings, kNeedsQuickdrawGX);
#endif	//	qRequireQuickDrawGX
#endif	//	qUseQuickDrawGX


	if ((Gestalt(gestaltAppleEventsAttr,&feature) == noErr) &&
		(feature & (1 << gestaltAppleEventsPresent)))
		{
		//	Figure out if we need to do AppleEvent recording
		gHasAppleScript = ((feature & (1 << gestaltScriptingSupport)) != 0);
		}
	else
		FatalErrorAlert(kCoreErrorStrings,kUnsupportedSystemSoftware);

#if	qInlineInputAware
	if ((Gestalt(gestaltTSMgrVersion,&feature) == noErr) && (feature >= 1))
		{
		gHasTextServices = true;
		if (Gestalt(gestaltTSMTEAttr, &feature) == noErr)
			gHasTSMTE = (feature & (1 << gestaltTSMTEPresent));

		if (InitTSMAwareApplication() != noErr)
			{
			gHasTextServices = false;
			gHasTSMTE = false;
			}
		}

#endif

	if (Gestalt(gestaltThreadMgrAttr,&feature) == noErr)
		{
#if GENERATINGCFM
		//	If running on a PowerPC, make sure that we not only have the
		//	68K Thread Manager, but also the PowerPC shared library, too.
		//	Because of the wonders of weak linking and out of memory errors
		//	we need to also check to make sure that an entrypoint in the library
		//	is there, too. 
		if ((Ptr) NewThread != (Ptr) kUnresolvedCFragSymbolAddress)
			gHasThreadManager = ((feature & ((1 << gestaltThreadMgrPresent) | (1 << gestaltThreadsLibraryPresent))) != 0);
#else
		gHasThreadManager = ((feature & (1 << gestaltThreadMgrPresent)) != 0);
#endif
		}

#if	qRequireThreadManager
	if (gHasThreadManager == false)
		FatalErrorAlert(kCoreErrorStrings, kNeedsThreadManager);
#endif

	//	Check for and install Drag Manager callbacks
	if (Gestalt(gestaltDragMgrAttr,&feature) == noErr)
		{
#if GENERATINGCFM
		//	If running on a PowerPC, make sure that we not only have the
		//	68K Drag Manager, but also the PowerPC shared library, too.
		if ((Ptr) NewDrag != (Ptr) kUnresolvedCFragSymbolAddress)
			gHasDragManager = ((feature & ((1 << gestaltDragMgrPresent) | (1 << gestaltPPCDragLibPresent))) != 0);
#else
		gHasDragManager = ((feature & (1 << gestaltDragMgrPresent)) != 0);
#endif

		if (gHasDragManager)
			{
			InstallTrackingHandler(NewDragTrackingHandlerProc(TWindow::CallDragTrackingHandler),(GrafPtr) nil,nil);
			InstallReceiveHandler(NewDragReceiveHandlerProc(TWindow::CallDragReceiveHandler),(GrafPtr) nil,nil);
			}
		}

	//	Check for Display Manager
	if (Gestalt(gestaltDisplayMgrAttr,&feature) == noErr)
		gHasDisplayManager = ((feature & (1 << gestaltDisplayMgrPresent)) != 0);

#if	qAOCEAware	
	//	Check for and initialize AOCE Standard Mail package if it exists
	if ((Gestalt(gestaltSMPMailerVersion,&feature) == noErr) && (feature != 0))
		{
#if GENERATINGCFM
		if ((Ptr) SMPInitMailer != kUnresolvedCFragSymbolAddress)
			gHasAOCE = (SMPInitMailer(kSMPVersion) == noErr);
#else
		gHasAOCE = (SMPInitMailer(kSMPVersion) == noErr);
#endif
		}
#endif

	//	Initialize our AppleEvent Handling Routines
	InitAppleEventRoutines();

	//	Setup desktop rectangle for dragging windows around			
	GetWMgrPort(&gWindowManagerPort);
	gDeskRectangle = (**GetGrayRgn()).rgnBBox;

	//	Setup the default menubar
	//		First, create a TMenuBar so we can register commands
	gMenuBar = new TMenuBar();

	//		Add the Macintosh menus via an MBAR resource
	gMenuBar->GetNewMenuBar(rMenuBar);
	
	//		add AppleMenu items
	AppendResMenu(GetMenuHandle(mApple),'DRVR');

	//		and add fonts to the font menu (if it is installed)
	if (GetMenuHandle(mFont))
		AppendResMenu(GetMenuHandle(mFont),'FONT');

	if (SetupApplication() == noErr)
		{
		delete gSplashWindow;	//	get rid of the splash screen
		MainEventLoop();
		TearDownApplication();
		}

#if	qInlineInputAware
	if (gHasTextServices)
		(void) CloseTSMAwareApplication();
#endif
		
#if	qUseQuickDrawGX
	//	Tear down QuickDrawGX
	if (gHasQuickDrawGX && gQuickDrawGXClient)
		{
		DisposeCommonColors();
		GXExitPrinting();		// Close the new printing mgr. 
		GXExitGraphics();		// Deallocate all of the default structures
		GXDisposeGraphicsClient(gQuickDrawGXClient);	
		}
#endif
	}


void
MainEventLoop(void)
	{
	EventRecord		anEvent;
	unsigned long	nextTimeToCheckForEvents = 0;
	
	while (!gDone)
		{
		if (gHasThreadManager)
			YieldToAnyThread();

		if ((gRunQuantum == 0) ||
			(TickCount() > nextTimeToCheckForEvents))
			{
			nextTimeToCheckForEvents = TickCount() + gRunQuantum;
			
			(void) WaitNextEvent(everyEvent,&anEvent,gSleepQuantum,gMouseRegion);
						
#if	qInlineInputAware
			if (gHasTextServices && TSMEventWrapperForKotoeri(&anEvent))
				{
				}
			else
#endif
				HandleEvent(&anEvent);
			}
		}
	}

#if	qInlineInputAware

Boolean
TSMEventWrapperForKotoeri(EventRecord *anEvent)
	{
	short		oldFont;
	ScriptCode	keyboardScript;
	Boolean		didTSMHandleTheEvent = false;

	//	This snippet of code is from TE 27: Inline Input & TextEdit
	//	and is related to working around a bug in Kotoeri, one of the
	//	Japanese Input methods when operating in a mixed script
	//	environment (e.g., Japanese Language Kit)
	
	// make sure we have a port and it's not the Window Manager port
	if (qd.thePort != nil)
		{
		oldFont			= qd.thePort->txFont;
		keyboardScript	= GetScriptManagerVariable(smKeyScript);

		if (FontToScript(oldFont) != keyboardScript)
			TextFont(GetScriptVariable(keyboardScript, smScriptAppFond));
		}

	didTSMHandleTheEvent = TSMEvent(anEvent);
	
	TextFont(oldFont);		//	restore the font, even though the sample code didnt

	return didTSMHandleTheEvent;
	}

#endif	



void
HandleEvent(EventRecord *anEvent)
	{
	TWindow	* wobj;
	
	if (anEvent->what != updateEvt)
		wobj = GetWindowObject(TWindow::FrontNonFloatingWindow());
	else
		wobj = GetWindowObject((WindowRef) anEvent->message);

	if (wobj != nil)
#if	qInlineInputAware
		// give text services a chance to set the cursor shape
		if (gHasTextServices && SetTSMCursor(anEvent->where))
			;	// do nothing, TSM did it for me
		else	
#endif
			wobj->AdjustCursor(anEvent);
	
	if ((wobj != nil) && wobj->EventFilter(anEvent))
		return;
		
	else switch (anEvent->what)
		{
		case	nullEvent:
			if (wobj != nil)
				wobj->Idle(anEvent);
			break;
			
		case	mouseDown:
			HandleMouseDown(wobj,anEvent);
			break;
		
		case	keyDown:
		case	autoKey:
			if (anEvent->modifiers & cmdKey)
				{
				if (wobj != nil)
					wobj->AdjustMenusBeforeMenuSelection();

				long	menuResult = MenuKey((short) anEvent->message & charCodeMask);
#if	qInlineInputAware
				if (!(gHasTextServices && TSMMenuSelect(menuResult)))
#endif
					HandleMenu(wobj,menuResult);
				if (wobj != nil)
					wobj->AdjustMenusAfterMenuSelection();
				HiliteMenu(0);
				}
			else if (wobj != nil)
				wobj->KeyDown(anEvent);
			break;
			
		case	updateEvt:
			{
			GrafPtr		oldPort;
			WindowRef	aWindow = (WindowRef) anEvent->message;	
				
			GetPort(&oldPort);
			SetPortWindowPort(aWindow);
			BeginUpdate(aWindow);
			if (wobj != nil)
				wobj->Draw();
			EndUpdate(aWindow);
			SetPort(oldPort);
			}
			break;
			
		case diskEvt:
			if (anEvent->message >> 16)
				{
				static	Point	where = {50,50};
				(void) DIBadMount(where,anEvent->message);
				}
			break;
				
		case	osEvt:
			switch ((anEvent->message & osEvtMessageMask) >> 24)
				{
				case	mouseMovedMessage:
					break;
					
				case	suspendResumeMessage:					

					TWindow::SuspendResumeWindows((anEvent->message & resumeFlag) != 0);

					if (anEvent->message & resumeFlag)
						{
						if (anEvent->message & convertClipboardFlag)
							ReadLocalClipboardFromScrap();

						gRunQuantum = gForegroundRunQuantum;
						gSleepQuantum = gForegroundSleepQuantum;
						}
					else
						{
						if (anEvent->message & convertClipboardFlag)
							WriteLocalClipboardToScrap();
						gRunQuantum = gBackgroundRunQuantum;
						gSleepQuantum = gBackgroundSleepQuantum;
						}

					break;
				}
			break;
		
		case	kHighLevelEvent:
			(void) AEProcessAppleEvent(anEvent);
			break;
			
		default:
			break;
		}
	}

void
HandleMouseDown(TWindow * topWindowObj,EventRecord *anEvent)
	{
	WindowRef	aWindow;
	TWindow		*wobj;
	short		partCode;

	partCode = FindWindow(anEvent->where,&aWindow);
	wobj = GetWindowObject(aWindow);
	switch(partCode)
		{
		case	inMenuBar:
			//	Thanks mclow! FindWindow made aWindow NULL when inMenuBar!
			wobj = GetWindowObject(TWindow::FrontNonFloatingWindow());
			if (wobj != nil)
				wobj->AdjustMenusBeforeMenuSelection();

			long	menuResult = MenuSelect(anEvent->where);
#if	qInlineInputAware
			if (!(gHasTextServices && TSMMenuSelect(menuResult)))
#endif
				HandleMenu(topWindowObj,menuResult);
			HiliteMenu(0);
			break;
			
		case	inSysWindow:
			SystemClick(anEvent, (GrafPtr) aWindow);
			break;
			
		case	inContent:
			if (wobj != nil)
				{
				GrafPtr	oldPort;
				
				GetPort(&oldPort);
				SetPortWindowPort(aWindow);
				GlobalToLocal(&anEvent->where);
				wobj->Click(anEvent);
				SetPortWindowPort(aWindow);
				}
			break;
			
		case	inDrag:
			if (wobj)
				wobj->Drag(anEvent->where);
			break;
			
		case	inGrow:
			if (wobj)
				wobj->Grow(anEvent->where);
			break;
			
		case	inGoAway:
			if (TrackGoAway(aWindow,anEvent->where))
				HandleClose(aWindow);
			break;

		case	inZoomIn:
    	case	inZoomOut:
    		if (TrackBox(aWindow,anEvent->where,partCode) && (wobj))
				wobj->Zoom(partCode);
    		break;
			
		default:
			break;
		}
	}
	

void
HandleClose(WindowRef aWindow)
	{
	short	windowKind;
	TWindow	*wobj;
	
	if (aWindow)
		{
		windowKind = GetWindowKind(aWindow);
		if (windowKind < 0)
			{
			CloseDeskAcc(GetWindowKind(aWindow));
			}
		else if ( ((wobj = GetWindowObject(aWindow)) != nil) &&
					wobj->CanClose() )
			{
			wobj->Close();
			}
		}
	}

void
HandleMenu(TWindow * topWindow, long menuCode)
	{
	short	menu = (short) (menuCode >> 16);
	short	item = (short) (menuCode & 0xffff);
					
	//	First, some special case menu handling for Mac OS:	
	//		 Openning Desk Accessories
	//		 Passing Edit commands through SystemEdit.

#if	!STRICT_MENUS
	if (menuCode == 0)
		{
		//	Check for selecting a hierarchical menus parent item
		
		menuCode = MenuChoice();
		if (menuCode != 0)
			{
			MenuHandle	theMenuHandle;
			short		cmdChar;

			menu = (short) (menuCode >> 16);
			item = (short) (menuCode & 0xffff);

			theMenuHandle = GetMenuHandle(menu);
			if (theMenuHandle != NULL)
				{
				//	If it wasnt a hierarchical menu parent, bail

				GetItemCmd(theMenuHandle,item,&cmdChar);
				if (cmdChar != hMenuCmd)
					return;

				//	If the item wasnt enabled, bail
		
				if (((**theMenuHandle).enableFlags & (1 << item)) == 0)
					return;
				}
			else
				return;
			}			
		}
#endif

	switch (menu)
		{
		case	mApple:
			if (item != iAbout)
				{
				Str255	deskAccName;
				
				GetMenuItemText(GetMenuHandle(mApple),item,deskAccName);
				(void) OpenDeskAcc(deskAccName);
				return;
				}
			break;
		
		case	mEdit:
			if (SystemEdit(item-1))
				return;
		
		default:
			break;
		}


	//	Next, check to see if the menu selection is a command

	MenuCommandID	command = gMenuBar->GetCommand(menu,item);

	if (command)
		{
		if (command == cQuit)
			{
			gDone = QuitApplication();
			return;
			}
		else if (topWindow)
			{
			if (topWindow->DoMenuCommand(command))
				return;
			}

		//	Nobody else wanted to handle the command, so let
		//	the application get a crack at it.
				
		HandleMenuCommand(command);
		}
	else
		{
		//	If we dont have a command, give the active window a
		//	chance to handle the raw menu selection.

		//	Windows which need to respond to dynamic menus (such as
		//	Font or Windows) menus should use this method.
		
		if (topWindow)
			{
			if (topWindow->DoMenuSelection(menu,item))
				return;
			}
		
		//	No window wanted to handle the selection, go ahead
		//	and give the application a crack at it.
		
		HandleMenuSelection(menu,item);
		}
	}
